"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HoistPeersWebpackResolvePlugin = void 0;
const path_1 = __importDefault(require("path"));
class HoistPeersWebpackResolvePlugin {
    constructor(options) {
        this.options = options;
        this.name = 'HoistPeersWebpackResolvePlugin';
    }
    apply(resolver) {
        // webpack v4 polyfill options.
        const resolveOptions = resolver.options || this.options.options;
        const { symlinks, modules, roots } = resolveOptions;
        const { deps = [] } = this.options;
        const projectRoot = Array.from(roots)[0];
        const nodeModulesBasePath = modules.find(module => typeof module === 'string' && path_1.default.isAbsolute(module) && module.endsWith('node_modules'));
        if (!nodeModulesBasePath || !nodeModulesBasePath.startsWith(projectRoot))
            throw new Error(`valid node_modules not found !`);
        const ignoreList = deps.map(depName => {
            var _a;
            const expectPkgJsonPath = path_1.default.join(nodeModulesBasePath, `${depName}/package.json`);
            const expectPkgDir = path_1.default.dirname(expectPkgJsonPath);
            const realPkgDir = path_1.default.dirname(require.resolve(expectPkgJsonPath));
            const isSymlink = symlinks && expectPkgDir !== realPkgDir;
            return {
                depName,
                isSymlink,
                expectPkgDir,
                realPkgDir,
                peers: Object
                    .keys(((_a = require(expectPkgJsonPath)) === null || _a === void 0 ? void 0 : _a.peerDependencies) || {})
                    .map(peerName => path_1.default.join(isSymlink ? realPkgDir : expectPkgDir, `node_modules/${peerName}`))
            };
        });
        resolver.getHook('existingFile').tapAsync(this.name, function (request, _resolveContext, callback) {
            try {
                const matchedDep = ignoreList.find(item => item.peers.some(peer => request.path && request.path.startsWith(peer)));
                if (matchedDep) {
                    const { isSymlink, expectPkgDir, realPkgDir } = matchedDep;
                    const rawRequestPath = request.path;
                    request.path = rawRequestPath.replace(path_1.default.join(isSymlink ? realPkgDir : expectPkgDir, 'node_modules'), nodeModulesBasePath);
                }
                callback();
            }
            catch (err) {
                callback(err, request);
            }
        });
    }
}
exports.HoistPeersWebpackResolvePlugin = HoistPeersWebpackResolvePlugin;
exports.default = HoistPeersWebpackResolvePlugin;
